@@ -1,6 +1,7 @@
 import logging
 import traceback
 import requests
+import orjson
 from .base import BaseHook
 
 
@@ -8,10 +9,10 @@ logger = logging.getLogger("polemarch")
 
 
 class Backend(BaseHook):
-    def execute(self, url, when, message) -> str:
+    def execute(self, url, when, message) -> str:  # pylint: disable=arguments-renamed
         data = dict(type=when, payload=message)
         try:
-            response = requests.post(url, json=data)
+            response = requests.post(url, data=orjson.dumps(data))  # pylint: disable=no-member
             return "{} {}: {}".format(
                 response.status_code, response.reason, response.text
             )
