@@ -5,18 +5,44 @@ import logging
 
 from django.contrib.auth.models import Group as BaseGroup
 from django.contrib.auth import get_user_model
-from .base import models, BModel, ACLModel, BQuerySet
+from django.db import models
+from vstutils.models import BModel, BQuerySet
+from .base import ACLModel
+from ..constants import MemberType
 
 
 logger = logging.getLogger("polemarch")
 
+User = get_user_model()
+
+
+class UserGroup(BaseGroup, ACLModel):
+    objects = BQuerySet.as_manager()
+    parent = models.OneToOneField(BaseGroup, on_delete=models.CASCADE, parent_link=True)
+    users = BaseGroup.user_set
+
+
+class ACLPermissionQuerySet(BQuerySet):
+    def filter_by_user(self, user):  # noce
+        return self.filter(models.Q(user=user) | models.Q(uagroup__id__in=user.groups.values('id')))
+
 
 class ACLPermission(BModel):
-    role    = models.CharField(max_length=10)
-    uagroup = models.ForeignKey('main.UserGroup',
-                                on_delete=models.CASCADE, blank=True, null=True)
-    user    = models.ForeignKey(get_user_model(),
-                                on_delete=models.CASCADE, blank=True, null=True)
+    objects = ACLPermissionQuerySet.as_manager()
+
+    role = models.CharField(max_length=10)
+    uagroup = models.ForeignKey(
+        UserGroup,
+        on_delete=models.CASCADE,
+        blank=True,
+        null=True,
+    )
+    user = models.ForeignKey(
+        User,
+        on_delete=models.CASCADE,
+        blank=True,
+        null=True,
+    )
 
     @property
     def member(self) -> int:  # noce
@@ -25,22 +51,8 @@ class ACLPermission(BModel):
             return self.user.id
         return self.uagroup.id
 
-    @member.setter
-    def member(self, value) -> None:  # nocv
-        pass
-
     @property
     def member_type(self) -> Text:  # noce
         if self.user is not None:
-            return "user"
-        return "team"
-
-    @member_type.setter
-    def member_type(self, value) -> None:  # nocv
-        pass
-
-
-class UserGroup(BaseGroup, ACLModel):
-    objects = BQuerySet.as_manager()
-    parent = models.OneToOneField(BaseGroup, on_delete=models.CASCADE, parent_link=True)
-    users = BaseGroup.user_set
+            return MemberType.USER
+        return MemberType.TEAM
