@@ -1,31 +1,94 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+
+import contextlib
 import threading
 from typing import Any, Iterable, Type, Union, Optional
-import os
 import time
-import signal
 import shutil
 import logging
 import tempfile
 import traceback
+import signal
 from pathlib import Path
 from collections import OrderedDict
 from subprocess import Popen
 from django.apps import apps
 from django.utils import timezone
-from vstutils.utils import KVExchanger
+from django.conf import settings
+from rest_framework.exceptions import ValidationError
+from vstutils.utils import ObjectHandlers, translate as _
+from vstutils.api import fields as vstfields
 
 from .hosts import Inventory
-from .tasks import History, Project
+from .history import History, Project
 from ...main.utils import CmdExecutor
-from ...plugins.base import BasePlugin
+from ...plugins.history.base import BasePlugin as BaseHistoryPlugin
+from ...plugins.execution.base import BasePlugin
+from ..executions import PLUGIN_HANDLERS
+from ..exceptions import IncompatibleError
+from ...api.fields import InventoryAutoCompletionField
 
 
 logger = logging.getLogger("polemarch")
 
 
-# Classes and methods for support
+def validate_inventory_arguments(execution_plugin, arguments: dict, project):
+    inventory_field_names = tuple(
+        # pylint: disable=protected-access
+        key for key, value in PLUGIN_HANDLERS.get_serializer_class(execution_plugin)._declared_fields.items()
+        if (
+            isinstance(value, InventoryAutoCompletionField)
+            or (isinstance(value, vstfields.FkModelField) and isinstance(value.select_model, Inventory))
+            or (isinstance(value, vstfields.FkField) and (
+                isinstance(value.select_model, Inventory) or value.select_model == 'Inventory',
+            ))
+        )
+    )
+    compatible_inventory_plugins = PLUGIN_HANDLERS.get_compatible_inventory_plugins(execution_plugin)
+    invalid_fields = set(compatible_inventory_plugins.keys()) - set(inventory_field_names)
+    if invalid_fields:
+        logger.warning(
+            f'Execution plugin {execution_plugin} has invalid fields for "compatible_inventory_plugins" '
+            f'option: {invalid_fields}'
+        )  # nocv
+    for field_name in inventory_field_names:
+        field_value = arguments.get(field_name)
+        if field_value is None:
+            continue
+        if isinstance(field_value, int):
+            field_value = Inventory.objects.get(id=field_value)
+        if isinstance(field_value, Inventory):
+            if not project.inventories.filter(id=field_value.id).exists():
+                raise ValidationError('No Inventory matches the given query.')
+            if field_value.plugin not in compatible_inventory_plugins.get(field_name, ()):
+                raise IncompatibleError(_('Field "{}" is not compatible with {} inventory plugin.').format(
+                    field_name,
+                    field_value.plugin,
+                ))
+            arguments[field_name] = field_value.id
+
+
+class HistoryPluginHandler(ObjectHandlers):
+    __slots__ = ()
+
+    def __init__(self):
+        super().__init__('HISTORY_PLUGIN_SETTINGS')
+
+    def get_writers(self, history: History) -> Iterable[BaseHistoryPlugin]:
+        for name in self.list():
+            if name not in settings.HISTORY_OUTPUT_PLUGINS:
+                continue  # nocv
+            backend = self[name]
+            if backend.writeable:
+                yield backend(history, **self.opts(name))
+
+    def get_reader(self, history: History) -> Optional[BaseHistoryPlugin]:
+        backend = self[settings.HISTORY_READ_PLUGIN]
+        if backend.readable:
+            return backend(history, **self.opts(settings.HISTORY_READ_PLUGIN))
+
+
 class DummyHistory:
     # pylint: disable=unused-argument
     def __init__(self, *args, **kwargs):
@@ -58,13 +121,21 @@ class DummyHistory:
 
 
 class Executor(CmdExecutor):
-    __slots__ = ('history', 'counter', 'exchanger', 'notificator',  'notificator_lock', 'notification_last_time')
+    __slots__ = (
+        'history',
+        'counter',
+        'notificator',
+        'notificator_lock',
+        'notification_last_time',
+        'must_die',
+        'writers',
+    )
 
-    def __init__(self, history: Union[History, DummyHistory]):
+    def __init__(self, history: Union[History, DummyHistory], writers: tuple = ()):
         super().__init__()
         self.history = history
         self.counter = 0
-        self.exchanger = KVExchanger(self.CANCEL_PREFIX + str(self.history.id))
+        self.writers = writers
         if isinstance(history, DummyHistory):
             self.notificator = None
         else:
@@ -73,6 +144,12 @@ class Executor(CmdExecutor):
             self.notificator_lock = threading.Lock()
             self.notification_last_time = 0
 
+        self.must_die = False
+        signal.signal(signal.SIGTERM, self.signal_handler)
+
+    def signal_handler(self, signum, frame):
+        self.must_die = True  # nocv
+
     @property
     def output(self) -> str:
         # Optimize for better performance.
@@ -83,20 +160,9 @@ class Executor(CmdExecutor):
         pass  # nocv
 
     def working_handler(self, proc: Popen):
-        if proc.poll() is None and self.exchanger.get() is not None:  # nocv
-            self.write_output("\n[ERROR]: User interrupted execution")
-            self.exchanger.delete()
-            for _ in range(5):
-                try:
-                    os.kill(-proc.pid, signal.SIGTERM)
-                except Exception:  # nocv
-                    break
-                proc.send_signal(signal.SIGINT)
-                time.sleep(5)
+        if proc.poll() is None and self.must_die:  # nocv
             proc.terminate()
-            proc.kill()
             proc.wait()
-        super().working_handler(proc)
         if self.notificator and time.time() - self.notification_last_time > 1:
             with self.notificator_lock:
                 if self.notificator.queue:
@@ -105,7 +171,8 @@ class Executor(CmdExecutor):
 
     def write_output(self, line: str):
         self.counter += 1
-        self.history.write_line(line, self.counter, '\n')
+        for writer in self.writers:
+            writer.write_line(line, self.counter, '\n')
         if self.notificator:
             with self.notificator_lock:
                 self.notificator.create_notification_from_instance(self.history)
@@ -113,7 +180,11 @@ class Executor(CmdExecutor):
     def execute(self, cmd: Iterable[str], cwd: str, env_vars: dict):
         self.env = env_vars
         pm_ansible_path = ' '.join(self.pm_ansible())
-        self.history.raw_args = " ".join(cmd).replace(pm_ansible_path, '').lstrip()
+        plain_command = ' '.join(cmd).replace(pm_ansible_path, '').lstrip()
+        if not self.history.raw_args:
+            self.history.raw_args = plain_command
+        else:
+            self.history.raw_args += f' && {plain_command}'
         ret = super().execute(cmd, cwd)
         if self.notificator:
             self.notificator.disconnect_all()
@@ -153,47 +224,59 @@ class PluginExecutor:
         'history',
         'raw_exec_args',
         'verbose_level',
-        '__execution_dir__',
+        'execution_dir',
         'plugin',
+        'plugin_name',
         'executor',
+        'writers',
     )
 
     executor_class = Executor
+    history_plugins = HistoryPluginHandler()
 
     def __init__(
         self,
+        plugin_name: str,
         plugin_class: Type[BasePlugin],
         plugin_options: dict,
         project: Project,
         history: Optional[History],
         exec_args,
     ):
+        # pylint: disable=too-many-arguments
         self.project = project
         self.history = history or DummyHistory()
         self.raw_exec_args = exec_args
-        self.__execution_dir__ = None
-        self.plugin = plugin_class(plugin_options, ProjectProxy(project), self.verbose_output)
+        self.execution_dir = None
+        self.plugin_name = plugin_name
+        self.plugin = plugin_class(output_handler=self.verbose_output, options=plugin_options)
         self.verbose_level = self.plugin.get_verbose_level(exec_args)
 
     def execute(self) -> None:
         try:
+            self.writers = tuple(self.history_plugins.get_writers(self.history))
             revision = self.get_execution_revision()
-            self.history.revision = revision or 'NO VCS'
-            self.project.repo_class.make_run_copy(str(self.execution_dir), revision)
-
-            self.executor = self.executor_class(self.history)
-            cmd, env_vars, self.history.raw_inventory = self.plugin.get_execution_data(
+            self.execution_dir = self.create_execution_dir()
+            self.history.revision = self.project.repo_class.make_run_copy(str(self.execution_dir), revision)
+            self.executor = self.executor_class(self.history, writers=self.writers)
+            cmd, env_vars = self.plugin.get_execution_data(
                 self.execution_dir,
-                self.raw_exec_args
+                self.raw_exec_args,
+                project_data=ProjectProxy(self.project),
             )
-
+            self.history.raw_inventory = self.plugin.get_raw_inventory()
             self.history.status = 'RUN'
             self.history.save()
+            self.send_hook('on_execution', execution_dir=str(self.execution_dir))
+
+            for pre_cmd in self.plugin.get_pre_commands(self.raw_exec_args):
+                self.verbose_output(f'Executing pre-command {pre_cmd}')
+                self.executor.execute(pre_cmd, str(self.execution_dir), env_vars)
 
-            self.history.status = 'OK'
-            self.send_hook('on_execution', execution_dir=self.execution_dir)
             self.verbose_output(f'Executing command {cmd}')
             self.executor.execute(cmd, str(self.execution_dir), env_vars)
+            self.history.status = 'OK'
+            self.plugin.post_execute_hook(cmd, self.raw_exec_args)
 
         except Exception as exception:
             logger.error(traceback.format_exc())
@@ -202,6 +285,9 @@ class PluginExecutor:
         finally:
             self.history.stop_time = timezone.now()
             self.history.save()
+            for writer in self.writers:
+                with contextlib.suppress(Exception):
+                    writer.finalize_output()
             self.send_hook('after_execution')
             self.__del__()
 
@@ -216,24 +302,18 @@ class PluginExecutor:
         elif isinstance(exception, self.project.SyncError):
             raise exception
 
-        last_line_object = self.history.raw_history_line.last()
-        last_line = 0
-        if last_line_object:
-            last_line = last_line_object.line_number  # nocv
+        last_line = self.history_plugins.get_reader(self.history).get_max_line()
         for line in error_text.split('\n'):
             last_line += 1
-            self.history.write_line(line, last_line)
+            for writer in self.writers:
+                writer.write_line(line, last_line)
 
-    @property
-    def execution_dir(self) -> Path:
-        if self.__execution_dir__ is not None:
-            return self.__execution_dir__
+    def create_execution_dir(self) -> Path:
         exec_dir = Path(tempfile.mkdtemp()) / 'execution_dir'
         self.verbose_output(f'Execution temp dir created: {exec_dir}')
-        self.__execution_dir__ = exec_dir
         return exec_dir
 
-    def send_hook(self, when: str, **kwargs) -> None:
+    def send_hook(self, when: str, **kwargs):
         msg = OrderedDict()
         msg['execution_type'] = self.history.kind
         msg['when'] = when
@@ -261,7 +341,7 @@ class PluginExecutor:
             logger.debug(message)
 
     def __del__(self):
-        exec_dir = getattr(self, '__execution_dir__', None)
+        exec_dir = getattr(self, 'execution_dir', None)
         if exec_dir is not None and exec_dir.is_dir():
             self.verbose_output(f'Temp dir {exec_dir} was cleared.')
             shutil.rmtree(exec_dir, ignore_errors=True)
