@@ -14,6 +14,8 @@ ENV_VARS_TYPE =  Dict[Text, Union[Text, bool]]  # pylint: disable=invalid-name
 
 
 class _VCS(_Base):  # nocv
+    __slots__ = ()
+
     def vcs_clone(self, *args, **kwargs):
         raise NotImplementedError()
 
@@ -25,7 +27,7 @@ class _VCS(_Base):  # nocv
 
 
 class Git(_VCS):
-    __slots__ = ('env', '_fetch_map',)
+    __slots__ = ('env', '_fetch_map', 'target_branch', 'persistent_config_options')
 
     _fetch_statuses = [
         "NEW_TAG", "NEW_HEAD", "HEAD_UPTODATE",
@@ -43,13 +45,17 @@ class Git(_VCS):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.env = self.options.get("GIT_ENV", {})
+        self.persistent_config_options = self.options.get("CONFIG_LIST", ())
         self._fetch_map = {
             1 << x: self._fetch_statuses[x] for x in range(8)
         }
         self.target_branch = self.proj.vars.get('repo_branch', None)
 
     def get_repo(self) -> git.Repo:
-        return git.Repo(self.path)
+        repo = git.Repo(self.path)
+        if self.persistent_config_options:
+            repo.git.set_persistent_git_options(c=self.persistent_config_options)
+        return repo
 
     def vcs_clone(self, source: str, destination, **kwargs) -> git.Repo:
         os.makedirs(destination)
@@ -60,6 +66,8 @@ class Git(_VCS):
                 **kwargs
             )
             repo = git.Repo(destination)
+            if self.persistent_config_options:
+                repo.git.set_persistent_git_options(c=self.persistent_config_options)
             with repo.git.custom_environment(**kwargs.get('env', {})):
                 self._update_submodules(repo, kill_after_timeout=kwargs.get('kill_after_timeout'))
         except git.GitCommandError as error:
@@ -166,7 +174,7 @@ class Git(_VCS):
                     config_writer.set_value("user", "email", self.proj.owner.email).release()
                     user_name = self.proj.owner.username
                     if self.proj.owner.last_name and self.proj.owner.first_name:  # nocv
-                        user_name = '{u.fist_name} {u.last_name}'.format(u=self.proj.owner)
+                        user_name = '{u.first_name} {u.last_name}'.format(u=self.proj.owner)
                     config_writer.set_value("user", "name", user_name).release()
                     repo.git.add(A=True)
                     repo.git.commit(m='Create project from Polemarch.')
@@ -180,11 +188,11 @@ class Git(_VCS):
 
         return (repo, result)
 
-    def get_revision(self, *args, **kwargs):
+    def get_revision(self, *args, repo=None, **kwargs):
         # pylint: disable=unused-argument
         if self.proj.status == 'NEW':
             return 'NOT_SYNCED'
-        repo = self.get_repo()
+        repo = repo or self.get_repo()
         return repo.head.object.hexsha
 
     def _with_password(self, tmp, env_vars: ENV_VARS_TYPE) -> ENV_VARS_TYPE:
@@ -248,11 +256,11 @@ class Git(_VCS):
         except git.GitError:
             return "ERROR"
 
-    def make_run_copy(self, destination: Text, revision: Text):
+    def make_run_copy(self, destination: Text, revision: Text) -> str:
         source = self.proj.path
         if self.proj.repo_sync_on_run:
             source = self.proj.repository
-        self._operate(
+        repo, _ = self._operate(
             self.make_clone,
             source=source,
             destination=str(destination),
@@ -260,3 +268,4 @@ class Git(_VCS):
             revision=revision,
             timeout=self.proj.repo_sync_timeout
         )
+        return self.get_revision(repo=repo)
